#lang scribble/manual

@; You should probably open artifact-guide/artifact-guide.html.

@(require
  scribble/examples
  (for-label
   "main.rkt"
   racket/base
   redex/reduction-semantics
   redex/gui
   racket/list
   racket/contract))

@title{Artifact: A low-level look at A-normal Form}
@author{@(author+email "William J. Bowman" "wjb@williamjbowman.com")}

@(define e (make-base-eval '(require racket/list)))

@defmodule["main.rkt" #:packages ()]{
This artifact guide is available as a PDF and in HTML.
The HTML version is recommended, as it features a more complete table of
contents, navigation bar, hyperlinking to additional online documentation, and
improved typesetting of example code.
}

@section{Hardware Dependencies}
Standard consumer laptop hardware should be sufficient.
An Intel or Arm 64-bit CPU is required to run standard builds of Racket, and at at least 4GBs of RAM to run some tests (2GB might suffice).
There is no required number of cores nor is a GPU required, nor a required operating system.

@section{Getting Started}
This artifact requires Racket, and has been tested in Racket 8.7 and above.
We recommend running this artifact locally, but it can also be run using the provided Docker files or QEMU image.
An installer can be downloaded for most major systems at @url{https://download.racket-lang.org}, including both Intel and Arm hardware.
It relies on the Redex package, which is included in the standard Racket install, but can be installed manually using @tt{raco pkg install redex}.

@;To run the Docker images, you can either build or import the image, then run @tt{docker run -it oopsla2024-159-artifact-amd64}.
@;To build the image using for your preferred platform using @emph{e.g.} @tt{docker build --tag oopsla2024-159-artifact-amd64 -f Dockerfile.amd64 --platform linux/amd64 .}.
@;Alternatively, load the prebuilt the image using @tt{docker image load -i oopsla2024-159-artifact-amd64-docker-image.tar}.
To run the Docker images, you can build the image using @emph{e.g.} @tt{docker build --tag oopsla2024-159-artifact-amd64 -f Dockerfile.amd64 --platform linux/amd64 .}, then run @tt{docker run -it oopsla2024-159-artifact-amd64}.
The artifact is stored in @tt{/artifact}, which should be the working directory for the Docker image.

To run the QEMU image, you can run @emph{e.g.} @tt{d2vm run qemu oopsla2024-159-artifact-amd64.qcow2} or @tt{qemu-system-x86_64 -m 4G oopsla2024-159-artifact-amd64.qcow2}.
The root password is @tt{root}.
The artifact is stored in @tt{/artifact}.

The main entry point for this artifact, which provides all languages, examples, reduction relations, and compilers defined in the paper, is @tt{main.rkt}.
This is largely implemented in @racketmodname[redex], but this documentation tries to provide sufficient usage instructions that knowledge of Redex is not required.

You can use the artifact by running expressions in the REPL after importing @tt{main.rkt}, or running any other module interactively with @racket[(require "main.rkt")], or running @tt{racket -i -t main.rkt} on the command line, or opening @tt{main.rkt} in DrRacket.
If running on a headless machine (including running a Docker container), you need to launch using @tt{xvfb-run}, such as @tt{xvfb-run racket -it main.rkt}, as Redex provides some optional GUI components.
Starting @tt{main.rkt} may take a minute as importing this module will compile and run the test suite.

There are minor differences between the paper and the implementation:
@itemize[
@item{The syntax for function application uses the keyword @tt{apply} rather
than juxtaposition. This means some rules in the paper are split into two rules in the artifact: one for operators with juxtaposition syntax, one for @tt{apply}.}
@item{@tt{let} requires a second pair of parenthesis, as in @racket[(let ([x 5]) x)].}
@item{@tt{if0} is instead written @tt{ifz}.}
@item{Fresh names may be generated or renamed differently.}
]

After installing Racket, run @tt{raco test ./} or @tt{xvfb-run raco test ./} in the same directory as @tt{main.rkt} to run the entire test suite; this should take several minutes.
This should end with @tt{55 tests passed}.
If it does, your installation should be working correctly.

To test interactively, run @tt{racket -i} or @tt{xvfb-run racket -i} in the same directory as @tt{main.rkt}, then run the following expressions in the REPL:
@examples[#:eval e
(require "main.rkt")
reg1-term
(cesk-max-stack (apply-reduction-relation* #:all? #t cesk-> (init-cesk (anf-compile reg1-term))))
(cesk-max-stack (apply-reduction-relation* #:all? #t cesk-> (init-cesk (abnormal-compile reg1-term))))
]

If you get the same return values as in the examples above, your installation should be working correctly.

@section{Introduction}
The theorems of the paper are testable via the artifact, and supported by tests.
Step-by-step instructions are provided below to create and run additional tests.
The theorems are not mechanically verified; Redex is not a theorem prover.
However, conjuncts in theorems featuring existential quantification are proven via tests.
@itemlist[
@item{Theorem 3.1: A-normalization optimizes the stack: tested in @tt{3-machine-semantics.rkt} lines 189--218. See also @racket[test-theorem-3.1?].}
@item{Theorem 3.2: A-normalization is unsafe-for-scope: tested in @tt{3-machine-semantics.rkt} line 650--668; this proves the inequality is strict in some case. See also @racket[test-theorem-3.2?].}
@item{Theorem 3.3: B-normalization is safe-for-scope: tested in @tt{3-machine-semantics.rkt} line 664. See also @racket[test-theorem-3.3?].}
@item{Theorem 4.1: AB-normalization optimizes the stack: tested in @tt{4-ab-normal-form.rkt} lines 240--246; this proves the inequality is strict in some case. See also @racket[test-theorem-4.1?].}
@item{Theorem 4.2: AB-normalization is safe-for-scope: tested in @tt{4-ab-normal-form.rkt} lines 620--634; this proves the inequality is strict in some case. See also @racket[test-theorem-4.2?].}
]

In addition, all examples, machines, languages, and compilers are defined, so new examples can be run and implementations can be extended.
These are documented in @Secref{sec:resuability}

@section{Step-by-Step Instructions}

@defproc[(test-theorem-3.1? (C (redex-match? monadicL C))) #t]{
Theorem 3.1 claims that A-normalizing a λ term in B-normal form results in at least no more stack usage, and strictly less in some cases.
To test this, for @racket[monadicL] @racket[C], the following expression should return @racket[#t] (true):
@racketblock[
(>=
 (ck-max-stack (apply-reduction-relation* #:all? #t bnf-ck-> (init-ck C)))
 (ck-max-stack (apply-reduction-relation* #:all? #t anf-ck-> (init-ck (a-normalize C)))))
]

For convenience, the function @racket[test-theorem-3.1?] can be used.
The function will validate its input and output.
The function also tests the witness for strict equality case of the theorem.
@racketblock[
(>
 (ck-max-stack (apply-reduction-relation* #:cache-all? #t bnf-ck-> (init-ck (b-normalize running-example))))
 (ck-max-stack (apply-reduction-relation* #:cache-all? #t anf-ck-> (init-ck (a-normalize running-example)))))
]

@examples[#:eval e
(test-theorem-3.1? intro-example-monadic)
(test-theorem-3.1? (b-normalize fact-example))
]
}

@defproc[(test-theorem-3.2? (C (redex-match? bregionL C))) #t]{
Theorem 3.2 claims that A-normalizing a λr term in B-normal form
results in at least as much maximum memory and as many maximum regions in use,
and more in some cases.
To test this, for any @racket[bregionL] term @racket[C], the following expressions should return @racket[#t] (true):
@racketblock[
(>=
 (csk-max-memory (apply-reduction-relation* #:all? #t region-csk-> (init-csk (car (apply-reduction-relation* #:cache-all? #t ra->* C)))))
 (csk-max-memory (apply-reduction-relation* #:all? #t region-csk-> (init-csk C))))

(>=
 (csk-max-regions (apply-reduction-relation* #:all? #t region-csk-> (init-csk (car (apply-reduction-relation* #:cache-all? #t ra->* C)))))
 (csk-max-regions (apply-reduction-relation* #:all? #t region-csk-> (init-csk C))))
]

For convenience, the function @racket[test-theorem-3.2?] can be used.
The function will validate its input and output.
The function also tests the witnesses for the strict equality cases of the theorem:
@racketblock[
(>
 (csk-max-memory (apply-reduction-relation* #:all? #t region-csk-> (init-csk (car (apply-reduction-relation* #:cache-all? #t ra->* reg1-term)))))
 (csk-max-memory (apply-reduction-relation* #:all? #t region-csk-> (init-csk reg1-term))))

(>
 (csk-max-regions (apply-reduction-relation* #:all? #t region-csk-> (init-csk (car (apply-reduction-relation* #:cache-all? #t ra->* reg1-term)))))
 (csk-max-regions (apply-reduction-relation* #:all? #t region-csk-> (init-csk reg1-term))))
]

@examples[#:eval e
(test-theorem-3.2? bnf-reg1-term)
]
}

@defproc[(test-theorem-3.3? (C (redex-match? regionL e))) #t]{
Theorem 3.3 claims that B-normalizing any a λr term
has no effect on maximum memory nor maximum regions in use.
To test this, for any @racket[regionL] @racket[e], the following expressions should return @racket[#t] (true):
@racketblock[
(=
 (csk-max-memory (apply-reduction-relation* #:all? #t region-csk-> (init-csk (car (apply-reduction-relation* #:cache-all? #t rb->* e)))))
 (csk-max-memory (apply-reduction-relation* #:all? #t region-csk-> (init-csk e))))

(=
 (csk-max-regions (apply-reduction-relation* #:all? #t region-csk-> (init-csk (car (apply-reduction-relation* #:cache-all? #t rb->* e)))))
 (csk-max-regions (apply-reduction-relation* #:all? #t region-csk-> (init-csk e))))
]

For convenience, the function @racket[test-theorem-3.3?] can be used.
The function will validate its input and output.

@examples[#:eval e
(test-theorem-3.3? reg1-term)
]
}

@defproc[(test-theorem-4.1? (C (redex-match? imonadicL t))) #t]{
Theorem 4.1 claims that AB-normalizing an imperative monadic term results in at least no more stack usage, and strictly less in some cases.
To test this, for any @racket[imonadicL] @racket[t], the following expressions should return @racket[#t] (true):
@racketblock[
(>=
 (cek-max-stack (apply-reduction-relation* #:all? #t cek-> (init-cek t)))
 (cek-max-stack (apply-reduction-relation* #:all? #t cek-> (init-cek (ab-normalize t)))))
]

For convenience, the function @racket[test-theorem-4.1?] can be used.
The function will validate its input and output.
The function also tests the witness for the strict equality case of the theorem.
@racketblock[
(>
 (cek-max-stack (apply-reduction-relation* #:all? #t cek-> (init-cek monadic-large-eg-term)))
 (cek-max-stack (apply-reduction-relation* #:all? #t cek-> (init-cek (ab-normalize monadic-large-eg-term)))))
]

@examples[#:eval e
(test-theorem-4.1? (monadic-code-gen (b-normalize nested-branches-example)))
]
}

@defproc[(test-theorem-4.2? (C (redex-match? rimonadicL t))) #t]{
Theorem 4.2 claims that AB-normalizing has no effect on maximum memory nor
maximum regions in use, and results in at least no more stack usage, and
strictly less in some cases.
To test this, for any @racket[rimonadicL] @racket[t], the following expressions should return @racket[#t] (true):
@racketblock[
(>=
 (cesk-max-stack (apply-reduction-relation* #:all? #t cesk-> (init-cesk t)))
 (cesk-max-stack (apply-reduction-relation* #:all? #t cesk-> (init-cesk (abnf t)))))

(=
 (cesk-max-memory (apply-reduction-relation* #:all? #t cesk-> (init-cesk t)))
 (cesk-max-memory (apply-reduction-relation* #:all? #t cesk-> (init-cesk (abnf t)))))

(=
 (cesk-max-regions (apply-reduction-relation* #:all? #t cesk-> (init-cesk t)))
 (cesk-max-regions (apply-reduction-relation* #:all? #t cesk-> (init-cesk (abnf t)))))
]

For convenience, the function @racket[test-theorem-4.2?] can be used.
The function will validate its input and output.
The function also tests the witness for the strict equality case of the theorem.
@racketblock[
(>
 (cesk-max-stack (apply-reduction-relation* #:all? #t cesk-> (init-cesk lst:code-gen-mregion-eg1)))
 (cesk-max-stack (apply-reduction-relation* #:all? #t cesk-> (init-cesk (abnf lst:code-gen-mregion-eg1)))))
]

@examples[#:eval e
(test-theorem-4.2? (abnf (monadic-code-gen bnf-reg1-term)))
]
}

@section[#:tag "sec:resuability"]{Reusability Guide}

Complete interface and usage documentation is provided in later sections.
The following list maps figures and listing from the paper to file and line numbers in the implementation, and briefly describes how models can be extended.
@itemlist[
@item{Listing 1: @tt{1-lc-syntax.rkt,} line 40 defines the listing as @racket[intro-example].}
@item{Listing 2: @tt{2-anf-and-monadic-form.rkt,} line 68 tests that Listing 1 A-normalizes to Listing 2.}
@item{Listing 3: @tt{3-machine-semantics.rkt,} line 716 defines the listing as @racket[listing3].}
@item{Listing 4: @tt{3-machine-semantics.rkt,} line 723 defines the listing as @racket[listing4]; line 466 tests that Listing 3 A-normalizes to Listing 4.}
@item{Listing 5: @tt{1-lc-syntax.rkt,} line 40 defines the listing as @racket[intro-example]; @tt{2-anf-and-monadic-form.rkt}, line 65 tests that the example is not in ANF.}
@item{Listing 6: @tt{1-lc-syntax.rkt,} line 45 defines the listing as @racket[intro-example-monadic]; @tt{2-anf-and-monadic-form.rkt}, line 66 tests that the example is not in ANF.}
@item{Listing 7: @tt{1-lc-syntax.rkt,} line 50 defines the listing as @racket[intro-example-anf]; @tt{2-anf-and-monadic-form.rkt}, line 64 tests that the example is in ANF.}
@item{Figure 1: @tt{1-lc-syntax.rkt} lines 6--38 defines the syntax of λ terms @racket[lambdaL] and @racket[eval-lambdaL]. @tt{2-anf-and-monadic.rkt}, line 11--43 defines the A-normalization reductions @racket[a->] and @racket[a->*].}
@item{Figure 2: @tt{2-anf-and-monadic.rkt,} line 45--55 defines the language @racket[ANFL].}
@item{Listing 8: @tt{1-lc-syntax.rkt,} line 56 defines the listing as @racket[running-example].}
@item{Listing 9: @tt{2-anf-and-monadic-form.rkt} line 74 tests that Listing 8 A-normalizes to Listing 9.}
@item{Listing 10: @tt{1-lc-syntax.rkt,} line 60 defines the listing as @racket[nested-branches-example].}
@item{Listing 11: @tt{2-anf-and-monadic-form.rkt} line 77 tests that Listing 10 A-normalizes to Listing 11.}
@item{Figure 3: @tt{2-anf-and-monadic-form.rkt} lines 137--145 define the syntax of monadic form as the language @racket[monadicL].}
@item{Figure 4: @tt{2-anf-and-monadic-form.rkt} lines 104--133 define the B-normalization for λ terms as @racket[b->] and @racket[b->*]. The artifact splits the application rule into 2 rules, one for built-in operators, and one for functions.}
@item{Listing 12: @tt{1-lc-syntax.rkt,} line 60 defines the listing as @racket[nested-branches-example].}
@item{Listing 13: @tt{2-anf-and-monadic-form.rkt} line 161 tests that Listing 12 B-normalizes to Listing 13.}
@item{Listing 14: @tt{2-anf-and-monadic-form.rkt} line 172 tests that Listing 12 B- then A-normalizes to Listing 14.}
@item{Figure 5: @tt{3-machine-semantics.rkt} lines 9--13 define the CK machine syntax as the language @racket[lambda-ckL], and 15--50 define the CK machine @racket[lambda-ck->].}
@item{Figure 6: @tt{3-machine-semantics.rkt} lines 115--154 define the ANF CK machine @racket[anf-ck->], and lines 67--110 define BNF CK machine @racket[bnf-ck->].}
@item{Figure 7: @tt{3-machine-semantics.rkt} lines 224--239 define the syntax λr as @racket[regionL].}
@item{Figure 8: @tt{3-machine-semantics.rkt} lines 282--335 define the CSK machine for λr as @racket[regionCSKL].}
@item{Inline definition: @tt{3-machine-semantics.rkt} lines 364--440 defines A-normalization for λr as @racket[ra->] and @racket[ra->*].}
@item{Listing 15: @tt{3-machine-semantics.rkt} line 448 defines the example as @racket[anf-reg1-term].}
@item{Listing 16: @tt{3-machine-semantics.rkt} line 602 defines the example as @racket[bnf-reg1-term].}
@item{Figure 9: @tt{4-ab-normal-form.rkt} lines 25--32 define the syntax of AB-normal form as @racket[abnfL].}
@item{Figure 10: @tt{4-ab-normal-form.rkt} lines 9--23 define the syntax of imperative monadic form as @racket[imonadicL].}
@item{Inline definition: @tt{4-ab-normal-form.rkt} lines 33--46 define the AB reductions as @racket[ab->] and @racket[ab->*].}
@item{Figure 11: @tt{4-ab-normal-form.rkt} lines 107--125 defines the CEK machine syntax as @racket[imonadic-cekL].}
@item{Figure 12: @tt{4-ab-normal-form.rkt} lines 127--199 defines the CEK machine for imperative monadic form as @racket[cek->].}
@item{Figure 13: @tt{4-ab-normal-form.rkt} lines 248--281 defines imperative monadic with regions syntax as @racket[rimonadicL].}
@item{Figure 14: @tt{4-ab-normal-form.rkt} lines 333--438 defines the CESK machine for the imperative monadic w/ regions syntax as @racket[cesk->].}
@item{Listing 17: @tt{4-ab-normal-form.rkt} line 659 generates the example for the listing.}
@item{Listing 18: @tt{4-ab-normal-form.rkt} line 660 generates the example for the listing.}
@item{Figure 16: @tt{5-compiler.rkt} lines 26--106 implement the ANF compiler pass with join points as @racket[anf].}
@item{Figure 17: @tt{5-compiler.rkt} lines 112--179 implement the monadic compiler pass as @racket[monadic].}
@item{Figure 18a: @tt{5-compiler.rkt} lines 207--250 implement the code generator from ANF as @racket[anf-code-gen].}
@item{Figure 18b: @tt{5-compiler.rkt} lines 252--281 implement the code generator from monadic form as @racket[monadic-code-gen].}
@item{Figure 19: @tt{5-compiler.rkt} lines 286--344 implement the AB-normalizer from imperative code as @racket[abnf].}
]

All syntax and languages can be extended by modifying or adding new nonterminals, following the documentation for @racket[define-language] and @racket[define-extended-language].
All reduction relations can be extended by modifying or adding new rules, following the documentation for @racket[reduction-relation] and @racket[extend-reduction-relation].
New examples and terms can be defined using @racket[define-term] or @racket[term]; see @racket[lambdaL] for examples using @racket[redex-match?] to check that the term is syntactically valid.

@subsection{λ, intro, examples, etc.}
@defthing[#:kind "language" lambdaL language?]{
The syntax of λ, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(redex-match?
 lambdaL
 e
 (term (apply (lambda (x) 5) 6)))

(redex-match?
 lambdaL
 ι
 '(apply (lambda (x) 5) 6))

(redex-match?
 lambdaL
 ι
 5)

(redex-match?
 lambdaL
 x
 'x1)
]
}

@defthing[#:kind "language" eval-lambdaL language?]{
The syntax of λ extended with evaluation contexts and values, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(redex-match?
 eval-lambdaL
 (in-hole E v)
 '(apply (lambda (x) 5) 6))

(redex-match
 eval-lambdaL
 (in-hole E v)
 '(apply (lambda (x) 5) 6))

(redex-match?
 eval-lambdaL
 E
 '(apply hole 6))
]
}

@defthing[#:kind "term" intro-example (redex-match? lambdaL e)]{
The λ term example from the intro.
}

@defthing[#:kind "term" intro-example-monadic (redex-match? lambdaL e)]{
The λ term example term in monadic form from the intro.
}

@defthing[#:kind "term" intro-example-anf (redex-match? lambdaL e)]{
The λ term example term in ANF from the intro.
}

@defthing[#:kind "term" running-example (redex-match? lambdaL e)]{
The λ term running example.
}

@defthing[#:kind "term" nested-branches-example (redex-match? lambdaL e)]{
The λ term example with nested case-of-case pattern.

@examples[#:eval e
nested-branches-example
]
}

@defthing[#:kind "term" fact-example (redex-match? lambdaL e)]{
A definition of the factorial function as a λ term.
}

@defthing[#:kind "language" lambda-ckL language?]{
The syntax of the λ CK machine as a Redex language (see @racket[define-language]).

@examples[#:eval e
(eval:check (redex-match? lambda-ckL K 'mt) #t)
(eval:check (redex-match? lambda-ckL K (term ((+ 4 hole) :: mt))) #t)
]

@margin-note{In Redex, @racket[term] behaves almost the same as @racket[quote],
but there are exceptions. For example, you must use @racket[term] to write a
term with a @racket[hole].}
}

@defproc[(init-ck [term (redex-match? lambdaL e)])
         (redex-match? lambda-ckL (e K))]{
Produces an initial machine configuration for the λ CK machine using @racket[term] for the initial code.
@examples[#:eval e
(init-ck intro-example)
]
}

@defthing[#:kind "reduction-relation" lambda-ck-> reduction-relation?]{
The λ CK abstract machine, as a Redex @racket[reduction-relation].
Try using with @racket[traces] to navigate the trace graphically.

@examples[#:eval e
(apply-reduction-relation* lambda-ck-> (init-ck intro-example))
(car (car (apply-reduction-relation* lambda-ck-> (init-ck fact-example))))
]
}

@subsection{A-normal form and Monadic form}
@defthing[#:kind "reduction-relation" a-> reduction-relation?]{
The single-step A-reductions for the @racket[lambdaL], as a Redex @racket[reduction-relation].
Try using with @racket[traces] to navigate the trace graphically.

This is non-determinisic due to fresh name generation.
The empty set of answers is returned when the program cannot single-step at the top-level.

@examples[#:eval e
(apply-reduction-relation a-> intro-example)
(car (apply-reduction-relation a-> intro-example))
]
}

@defthing[#:kind "reduction-relation" a->* reduction-relation?]{
The compatible closure of the @racket[a->], as a Redex @racket[reduction-relation].
Try using with @racket[traces] to navigate the trace graphically.

This is non-determinisic due to fresh name generation and (confluent) choices of evaluation contexts.

@examples[#:eval e
(apply-reduction-relation a->* intro-example)
(apply-reduction-relation* a->* #:cache-all? #t nested-branches-example)
]
}

@defproc[(a-normalize [term (redex-match? lambdaL e)])
         (redex-match? ANFL m)]{
A-normalizes @racket[term] by running @racket[a->*] to a normal form.
@examples[#:eval e
(a-normalize intro-example)
]
}

@defthing[#:kind "language" ANFL language?]{
The syntax of A-normal form, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(redex-match? ANFL M '(let ([x (apply f 1)]) (+ x 2)))
(redex-match? ANFL M '(+ (apply f 1) 2))
]
}


@defthing[#:kind "language" monadic-eval-lambdaL language?]{
The syntax of the λ extended with non-strict monadic evaluation contexts and values, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(redex-match? monadic-eval-lambdaL En (term (let ([x hole]) 5)))
(redex-match? eval-lambdaL E (term (let ([x hole]) 5)))

(redex-match? monadic-eval-lambdaL En (term (ifz hole 5 6)))
(redex-match? eval-lambdaL E (term (ifz hole 5 6)))
]
}

@defthing[#:kind "reduction-relation" b-> reduction-relation?]{
The single-step B-reductions for the @racket[lambdaL], as a Redex @racket[reduction-relation].
Try using with @racket[traces] to navigate the trace graphically.

This is non-determinisic due to fresh name generation and (confluent) choices of evaluation contexts.
The empty set of answers is returned when the program cannot single-step at the top-level.

@examples[#:eval e
(apply-reduction-relation b-> intro-example)
(apply-reduction-relation b-> nested-branches-example)
]
}

@defthing[#:kind "reduction-relation" b->* reduction-relation?]{
The compatible closure of the @racket[b->], as a Redex @racket[reduction-relation].
Try using with @racket[traces] to navigate the trace graphically.

This is non-determinisic due to fresh name generation and (confluent) choices of evaluation contexts.

@examples[#:eval e
(car (apply-reduction-relation b->* nested-branches-example))
(car (apply-reduction-relation* b->* nested-branches-example))
]
}

@defproc[(b-normalize [term (redex-match? lambdaL e)])
         (redex-match? monadicL C)]{
B-normalizes @racket[term] by running @racket[b->*] to a normal form.
@examples[#:eval e
(b-normalize nested-branches-example)
]
}

@defthing[#:kind "language" monadicL language?]{
The syntax of B-normal form, @emph{i.e.}, monadic form, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(redex-match? monadicL C '(let ([x (apply f 1)]) (+ x 2)))
(redex-match? monadicL C '(+ (apply f 1) 2))
(redex-match? monadicL C '(let ((y (let ([x (apply f 1)]) (+ x 2)))) y))
]
}

@subsection{ANF and Monadic Machines}

@defthing[#:kind "reduction-relation" bnf-ck-> reduction-relation?]{
The BNF (monadic form) CK abstract machine, as a Redex @racket[reduction-relation].
Try using with @racket[traces] to navigate the trace graphically.

@examples[#:eval e
(apply-reduction-relation* lambda-ck-> (init-ck running-example))
(apply-reduction-relation* bnf-ck-> (init-ck (b-normalize running-example)))
]
}

@defthing[#:kind "reduction-relation" anf-ck-> reduction-relation?]{
The ANF CK abstract machine, as a Redex @racket[reduction-relation].
Try using with @racket[traces] to navigate the trace graphically.

@examples[#:eval e
(apply-reduction-relation* lambda-ck-> (init-ck running-example))
(apply-reduction-relation* anf-ck-> (init-ck (a-normalize running-example)))
]
}

@defproc[(ck-max-stack [states (listof (redex-match? lambda-ckL (e K)))])
         natural-number?]{
Returns the maximum stack length found in a CK machine trace (either @racket[lambda-ck->] or @racket[anf-ck->]).

@examples[#:eval e
(ck-max-stack (apply-reduction-relation* #:all? #t anf-ck-> (init-ck (a-normalize running-example))))
(ck-max-stack (apply-reduction-relation* #:all? #t bnf-ck-> (init-ck (b-normalize running-example))))
(ck-max-stack (apply-reduction-relation* #:all? #t lambda-ck-> (init-ck running-example)))
]
}

@defthing[#:kind "language" regionL language?]{
The λ-region syntax, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(eval:check (redex-match? regionL e '(\@ r0 5)) #t)
(eval:check (redex-match? regionL e '(letregion r1 (\@ r0 5))) #t)
]
}

@defthing[#:kind "language" regionCSKL language?]{
The λ-region CSK abstract machine syntax, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(eval:check (redex-match? regionCSKL K (term ((free r1) :: mt))) #t)
(eval:check (redex-match? regionCSKL S '()) #t)
(eval:check (redex-match? regionCSKL S '(() (r0 ()))) #t)
(eval:check (redex-match? regionCSKL S '(() (r0 (() (o1 5))))) #t)
(eval:check (redex-match? regionCSKL a '(r0 o1)) #t)
]

@margin-note{In Redex, @racket[term] behaves almost the same as @racket[quote],
but there are exceptions. For example, you must use @racket[term] to write a
term with a @racket[hole].}
}

@defthing[#:kind "reduction-relation" region-csk-> reduction-relation?]{
The λ-region CSK abstract machine, as a Redex @racket[reduction-relation].
Try using with @racket[traces] to navigate the trace graphically.

@examples[#:eval e
(csk-read-val (apply-reduction-relation* region-csk-> (init-csk reg1-term)))
]
}

@defthing[reg1-term (redex-match? regionL e)]{
An example term for λ-regions.
}

@defproc[(init-csk [term (redex-match? regionL e)])
         (redex-match? regionCSKL (e S K))]{
Produces an initial machine configuration for the λ-region CSK machine using @racket[term] for the initial code, and an empty initial store and kontinuation.

@examples[#:eval e
(init-csk reg1-term)
]
}

@defproc[(csk-read-val [states (listof (redex-match? regionCSKL (e S K)))])
         (redex-match? regionL v)]{
Produces the final value of the CSK machine from its final configuration.
Expects that @racket[states] is a singleton set of terminal machine configurations, @emph{i.e.}, that the machine normalized to a single final state, whose code is a valid address and whose kontinuation is empty.

@examples[#:eval e
(csk-read-val '(((r0 o1) (() (r0 (() (o1 5)))) mt)))
]
}

@defproc[(csk-max-regions [trace (listof (redex-match? regionCSKL (e S K)))])
         natural-number?]{
Returns the maximum number of regions allocated in the @racket[region-csk->] trace.

@examples[#:eval e
(csk-max-regions (apply-reduction-relation* region-csk-> (init-csk reg1-term)))
(csk-max-regions (apply-reduction-relation* region-csk-> (init-csk anf-reg1-term)))
(csk-max-regions (apply-reduction-relation* region-csk-> (init-csk bnf-reg1-term)))
]
}

@defproc[(csk-max-memory [trace (listof (redex-match? regionCSKL (e S K)))])
         natural-number?]{
Returns the maximum number of live addresses in the @racket[region-csk->] trace.

@examples[#:eval e
(csk-max-memory (apply-reduction-relation* region-csk-> (init-csk reg1-term)))
(csk-max-memory (apply-reduction-relation* region-csk-> (init-csk anf-reg1-term)))
(csk-max-memory (apply-reduction-relation* region-csk-> (init-csk bnf-reg1-term)))
]
}

@defthing[#:kind "language" aregionL language?]{
The λ-regions syntax extended with evaluation contexts for A-reduction, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(eval:check (redex-match? aregionL E (term (\@ r hole))) #t)
(eval:check (redex-match? aregionL E (term (let ([x hole]) e))) #t)
]

@margin-note{In Redex, @racket[term] behaves almost the same as @racket[quote],
but there are exceptions. For example, you must use @racket[term] to write a
term with a @racket[hole].}
}

@defthing[#:kind "reduction-relation" ra-> reduction-relation?]{
The single-step A-reductions for λ-regions (@racket[aregionL]), as a Redex @racket[reduction-relation].

This is non-determinisic due to fresh name generation and (confluent) choices of evaluation contexts.
The empty set of answers is returned when the program cannot single-step at the top-level.

@examples[#:eval e
(apply-reduction-relation ra-> reg1-term)
]
}

@defthing[#:kind "reduction-relation" ra->* reduction-relation?]{
The compatible closure of @racket[ra->], as a Redex @racket[reduction-relation].

This is non-determinisic due to fresh name generation and (confluent) choices of evaluation contexts.
This can be rather slow due to non-determinism, so you probably want to enable @racket[#:cache-all?].

@examples[#:eval e
(take (apply-reduction-relation ra->* reg1-term) 3)
(car (apply-reduction-relation* ra->* #:cache-all? #t reg1-term))
]
}

@defthing[anf-reg1-term (redex-match? regionL e)]{
The A-normal form of @racket[reg1-term]
}

@defthing[#:kind "language" bregionL language?]{
The λ-regions syntax extended with non-strict monadic evaluation contexts for B-reduction, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(eval:check (redex-match? bregionL En (term (\@ r hole))) #t)
(eval:check (redex-match? bregionL En (term (let ([x hole]) e))) #f)
]

@margin-note{In Redex, @racket[term] behaves almost the same as @racket[quote],
but there are exceptions. For example, you must use @racket[term] to write a
term with a @racket[hole].}
}

@defthing[#:kind "reduction-relation" rb-> reduction-relation?]{
The single-step B-reductions for the λ-regions (@racket[aregionL]), as a Redex @racket[reduction-relation].

This is non-determinisic due to fresh name generation and (confluent) choices of evaluation contexts.
The empty set of answers is returned when the program cannot single-step at the top-level.

@examples[#:eval e
(apply-reduction-relation rb-> reg1-term)
]
}

@defthing[#:kind "reduction-relation" rb->* reduction-relation?]{
The compatible closure of @racket[rb->], as a Redex @racket[reduction-relation].

This is non-determinisic due to fresh name generation and (confluent) choices of evaluation contexts.
This can be rather slow due to non-determinism, so you probably want to enable @racket[#:cache-all?].

@examples[#:eval e
(take (apply-reduction-relation rb->* reg1-term) 3)
(car (apply-reduction-relation* rb->* #:cache-all? #t reg1-term))
]
}

@defthing[bnf-reg1-term (redex-match? regionL e)]{
The B-normal form of @racket[reg1-term]
}

@subsection{Imperative A-normalization, Machines, and AB-normalization}

@defthing[#:kind "language" imonadicL language?]{
The imperative monadic syntax, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(eval:check (redex-match? imonadicL t '(begin (set! x (begin (set! y 5) y)) x)) #t)
]
}

@defthing[#:kind "language" abnfL language?]{
The imperative ANF syntax, @emph{i.e.}, AB-normal form, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(eval:check (redex-match? imonadicL t '(begin (set! x (begin (set! y 5) y)) x)) #t)
(eval:check (redex-match? abnfL t '(begin (set! x (begin (set! y 5) y)) x)) #f)
(eval:check (redex-match? abnfL t '(begin (set! y 5) (set! x y) x)) #t)
]
}

@defthing[#:kind "reduction-relation" ab-> reduction-relation?]{
The single-step AB-reductions for the @racket[imonadicL] @emph{statements}, as a Redex @racket[reduction-relation].
Try using with @racket[traces] to navigate the trace graphically.

The empty set of answers is returned when the program cannot single-step at the top-level.

@examples[#:eval e
(apply-reduction-relation ab-> '(set! x (begin (set! y 5) y)))
]
}

@defthing[#:kind "reduction-relation" ab->* reduction-relation?]{
The compatible closure of @racket[ab->], as a Redex @racket[reduction-relation].
This lifts @racket[ab->] to work over tails, as well.
Try using with @racket[traces] to navigate the trace graphically.

@examples[#:eval e
(apply-reduction-relation ab->* '(begin (set! x (begin (set! y 5) y)) x))
(apply-reduction-relation ab->* (monadic-code-gen (b-normalize nested-branches-example)))
]
}

@defproc[(ab-normalize [term (redex-match? imonadicL t)])
         (redex-match? abnfL t)]{
AB-normalizes @racket[term] by running @racket[ab->*] to a normal form.
@examples[#:eval e
(ab-normalize (monadic-code-gen (b-normalize nested-branches-example)))
]
}

@defthing[#:kind "language" imonadic-cekL language?]{
The imperative CEK abstract machine syntax, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(eval:check (redex-match? imonadic-cekL K (term ((begin (set! x hole) x) :: mt))) #t)
(eval:check (redex-match? imonadic-cekL Σ (term (() (x 5)))) #t)
]

@margin-note{In Redex, @racket[term] behaves almost the same as @racket[quote],
but there are exceptions. For example, you must use @racket[term] to write a
term with a @racket[hole].}
}

@defthing[#:kind "reduction-relation" cek-> reduction-relation?]{
The imperative CEK abstract machine, as a Redex @racket[reduction-relation].
Try using with @racket[traces] to navigate the trace graphically.

@examples[#:eval e
(apply-reduction-relation* cek-> (init-cek '(begin (set! x (begin (set! y 5) y)) x)))
(car (car (apply-reduction-relation* cek-> (init-cek '(begin (set! x (begin (set! y 5) y)) x)))))
]
}

@defproc[(init-cek [term (redex-match? imonadicL e)])
         (redex-match? imonadic-cekL (e Σ K))]{
Produces an initial machine configuration for the monadic imperative CEK machine using @racket[term] for the initial code.
@examples[#:eval e
(init-cek '(begin (set! x 5) x))
]
}

@defproc[(cek-max-stack [states (listof (redex-match? imonadic-cekL (e Σ K)))])
          natural-number?]{
Returns the maximum stack length found in a @racket[cek->] trace.

@examples[#:eval e
(cek-max-stack
 (apply-reduction-relation* #:all? #t
  cek->
  (init-cek (monadic-code-gen (b-normalize nested-branches-example)))))
(cek-max-stack
 (apply-reduction-relation* #:all? #t
  cek->
  (init-cek (ab-normalize (monadic-code-gen (b-normalize nested-branches-example))))))
]
}

@defthing[monadic-large-eg-term (redex-match? imonadicL t)]{
An imperative monadic example with a large continuation after a branch.
}


@defthing[#:kind "language" rimonadicL language?]{
The imperative monadic with regions syntax, as a Redex language (see @racket[define-language]).

@examples[#:eval e
(eval:check (redex-match? rimonadicL t '(begin (set! x (begin (set! y 5) y)) x)) #f)
(eval:check (redex-match? rimonadicL t '(begin (set! x (begin (set! y (alloc r0 5)) y)) x)) #t)
]
}

@defthing[#:kind "reduction-relation" cesk-> reduction-relation?]{
The imperative CESK abstract machine, as a Redex @racket[reduction-relation].
Try using with @racket[traces] to navigate the trace graphically.

@examples[#:eval e
(apply-reduction-relation* cesk-> (init-cesk '(begin (set! x (begin (set! y (alloc r0 5)) y)) x)))
(cesk-read-val (apply-reduction-relation* cesk-> (init-cesk '(begin (set! x (begin (set! y (alloc r0 5)) y)) x))))
]
}

@defproc[(init-cesk [term (redex-match? rimonadicL t)])
         (redex-match? rimonadicL (t Σ S K))]{
Produces an initial machine configuration for the monadic imperative with regions CESK machine using @racket[term] for the initial code.
The machine starts with an empty environment, empty kontinuation, and an initial empty region @racket[r0] where the final result must be allocated.

@examples[#:eval e
(init-cesk '(begin (set! x (alloc r0 5)) x))
]
}

@defproc[(cesk-read-val [states (listof (redex-match? rimonadicL (t Σ S K)))])
         (redex-match? rimonadicL v)]{
Produces the final value of the CESK machine from its final configuration.
Expects that @racket[states] is a singleton set of terminal machine configurations, @emph{i.e.}, that the machine normalized to a single final state, whose code is a valid address and whose kontinuation is empty.

@examples[#:eval e
(cesk-read-val (apply-reduction-relation* cesk-> (init-cesk '(begin (set! x (alloc r0 5)) x))))
]
}

@defproc[(cesk-max-stack [states (listof (redex-match? rimonadicL (t Σ S K)))])
         natural-number?]{
Returns the maximum stack length found in a @racket[cesk->] trace.

@examples[#:eval e
(cesk-max-stack (apply-reduction-relation* #:all? #t cesk-> (init-cesk '(begin (set! x (alloc r0 5)) x))))
(define D_b
 (apply-reduction-relation* #:all? #t cesk->
  (init-cesk (monadic-code-gen bnf-reg1-term))))
(define D_a
 (apply-reduction-relation* #:all? #t cesk->
  (init-cesk (anf-code-gen anf-reg1-term))))
(define D_ab
 (apply-reduction-relation* #:all? #t cesk->
  (init-cesk (abnf (monadic-code-gen bnf-reg1-term)))))
(cesk-max-stack D_b)
(cesk-max-stack D_a)
(cesk-max-stack D_ab)
]

@margin-note{A minor bug exists in the ANF code generator for regions which unnecessarily introduces a stack frame. Note that this bug does not affect the @racket[cek-max-stack].}
}

@defproc[(cesk-max-memory [states (listof (redex-match? rimonadicL (t Σ S K)))])
         natural-number?]{
Returns the maximum number of live addresses in the @racket[cesk->] trace.

@examples[#:eval e
(cesk-max-memory (apply-reduction-relation* #:all? #t cesk-> (init-cesk '(begin (set! x (alloc r0 5)) x))))
(cesk-max-memory D_b)
(cesk-max-memory D_a)
(cesk-max-memory D_ab)
]
}

@defproc[(cesk-max-regions [states (listof (redex-match? rimonadicL (t Σ S K)))])
         natural-number?]{
Returns the maximum number of regions allocated in the @racket[cesk->] trace.

@examples[#:eval e
(cesk-max-regions (apply-reduction-relation* #:all? #t cesk-> (init-cesk '(begin (set! x (alloc r0 5)) x))))
(cesk-max-regions D_b)
(cesk-max-regions D_a)
(cesk-max-regions D_ab)
]
}

@defthing[lst:code-gen-mregion-eg1 (redex-match? rimonadicL t)]{
An imperative monadic with regions term with two independent regions.
}

@subsection{Compiler}
@defproc[(anf [term (or/c (redex-match? regionL e) (redex-match? lambdaL e))]
              [fresh (-> symbol? symbol?) gensym])
         (or/c (redex-match? regionL e) (redex-match? ANFL M))]{
Produces the A-normal form of a λ or λ-regions term.
Optionally takes a source of fresh names.

@examples[#:eval e
(anf intro-example)
(anf reg1-term)
(anf intro-example (inc-var))
]
}

@defproc[(monadic [term (or/c (redex-match? regionL e) (redex-match? lambdaL e))]
                  [fresh (-> symbol? symbol?) gensym])
         (or/c (redex-match? regionL e) (redex-match? monadicL C))]{
Produces the monadic form of a λ or λ-regions term.

@examples[#:eval e
(monadic intro-example)
(monadic reg1-term)
(monadic reg1-term (inc-var))
]
}

@defproc[(anf-code-gen [term (or/c (redex-match? regionL e) (redex-match? ANFL M))])
         (or/c (redex-match? rimonadicL t) (redex-match? abnfL t))]{
Genernate the CESK machine code of the ANF term @racket[term].
The output is AB-normal form if the input was A-normal.

@examples[#:eval e
(flatten-begin (anf-code-gen (anf intro-example)))
(flatten-begin (anf-code-gen (anf reg1-term (inc-var))))
]

@margin-note{A minor bug exists in the ANF code generator for regions which unnecessarily introduces a stack frame. Note that this bug does not affect the @racket[cek-max-stack].}
}
}

@defproc[(monadic-code-gen [term (or/c (redex-match? regionL e) (redex-match? monadicL C))])
         (or/c (redex-match? rimonadicL t) (redex-match? imonadicL t))]{
Genernate the CESK machine code of the monadic term @racket[term].

@examples[#:eval e
(flatten-begin (monadic-code-gen (monadic intro-example)))
(flatten-begin (monadic-code-gen (monadic reg1-term)))
]
}

@defproc[(abnf [term (or/c (redex-match? rimonadicL t) (redex-match? imonadicL t))])
         (or/c (redex-match? rimonadicL t) (redex-match? abnfL t))
]{
AB-normalize the CESK machine code of the monadic term @racket[term].

@examples[#:eval e
(abnf (monadic-code-gen (monadic intro-example)))
(flatten-begin (abnf (monadic-code-gen (monadic reg1-term))))
(flatten-begin (anf-code-gen (anf reg1-term)))
(flatten-begin (abnf (anf-code-gen (anf reg1-term))))
(flatten-begin (anf-code-gen (anf intro-example)))
(flatten-begin (abnf (anf-code-gen (anf intro-example))))
]
}

@defproc[(anf-compile [term (or/c (redex-match? regionL e) (redex-match? lambdaL e))]
                      [fresh (-> symbol? symbol?) gensym])
         (or/c (redex-match? rimonadic t) (redex-match? abnfL t))]{
Compile a term with the ANF compiler.
Optionally takes a source of fresh names.

@examples[#:eval e
(flatten-begin (anf-compile nested-branches-example (inc-var)))
]
}

@defproc[(abnormal-compile [term (or/c (redex-match? regionL e) (redex-match? lambdaL e))]
                           [fresh (-> symbol? symbol?) gensym])
         (or/c (redex-match? rimonadic t) (redex-match? abnfL t))]{
Compile a term with the AB-normal compiler.
Optionally takes a source of fresh names.

@examples[#:eval e
(flatten-begin (abnormal-compile nested-branches-example (inc-var)))
]
}

@defproc[(inc-var [init natural? 1])
         (-> symbol? symbol?)]{
A predictable name generator factory.
Optionally takes an starting number to append to a generated name.

May not actually generate fresh names, depending on the context in which it is used.

@examples[#:eval e
(define gsym (inc-var 2))
(gsym 'x)
(gsym 'x)
]
}

@defproc[(flatten-begin [term any/c]) any/c]{
Flatten all nested begins; normalizes the @tt{admin1} and @tt{admin2} transitions.
Not used in any of the compilers; useful for pretty-printing.

@examples[#:eval e
(flatten-begin '(begin (begin (set! x y) (begin (set! x 5))) x))]
}
